function unpack(rows, key) {
    return rows.map(function (row) {
        return row[key];
    });
}

document.addEventListener("DOMContentLoaded", function () {
    const params = new URLSearchParams(window.location.search)
    const pair = params.get("pair") || ""
    fetch("/data?pair="+pair).
    then(data => data.json())
    .then(data => {
        const candleStickData = {
            name: "Candles",
            x: unpack(data.candles, "time"),
            close: unpack(data.candles, "close"),
            open: unpack(data.candles, "open"),
            low: unpack(data.candles, "low"),
            high: unpack(data.candles, "high"),
            type: "candlestick",
            xaxis: "x1",
            yaxis: "y1",
        };

        const points = [];
        const annotations = [];
        data.candles.forEach((candle) => {
            candle.orders.forEach(order => {
                const point = {
                    time: candle.time,
                    position: order.price,
                    side: order.side,
                    color: "green"
                }
                if (order.side === "SELL") {
                    point.color = "red"
                }
                points.push(point);

                const annotation = {
                    x: candle.time,
                    y: candle.low,
                    xref: "x",
                    yref: "y",
                    xaxis: "x1",
                    yaxis: "y1",
                    text: "B",
                    hovertext: `${order.time}
                        <br>ID: ${order.id}
                        <br>Price: ${order.price.toLocaleString()}
                        <br>Size: ${order.quantity.toPrecision(4).toLocaleString()}
                        <br>Type: ${order.type}
                        <br>${(order.profit && "Profit: " + (order.profit * 100).toPrecision(2).toLocaleString() + "%") || ""}`,
                    showarrow: true,
                    arrowcolor: "green",
                    valign: "bottom",
                    borderpad: 4,
                    arrowhead: 2,
                    ax: 0,
                    ay: 20,
                    font: {
                        size: 12,
                        color: "green",
                    },
                };

                if (order.side === "SELL") {
                    annotation.font.color = "red";
                    annotation.arrowcolor = "red";
                    annotation.text = "S";
                    annotation.y = candle.high;
                    annotation.ay = -20;
                    annotation.valign = "top";
                }

                annotations.push(annotation);
            });
        });

        const sellPoints = points.filter(p => p.side === "SELL");
        const buyPoints = points.filter(p => p.side === "BUY");
        const buyData = {
            name: "Buy Points",
            x: unpack(buyPoints, "time"),
            y: unpack(buyPoints, "position"),
            xaxis: "x1",
            yaxis: "y1",
            mode: 'markers',
            type: 'scatter',
            marker: {
                color: "green",
            }
        };
        const sellData = {
            name: "Sell Points",
            x: unpack(sellPoints, "time"),
            y: unpack(sellPoints, "position"),
            xaxis: "x1",
            yaxis: "y1",
            mode: 'markers',
            type: 'scatter',
            marker: {
                color: "red",
            }
        };

        const standaloneIndicators = data.indicators.reduce((total, indicator) => {
            if (!indicator.overlay) {
                return total + 1;
            }
            return total;
        }, 0);

        let layout = {
            template: "ggplot2",
            dragmode: "zoom",
            margin: {
                t: 25,
            },
            showlegend: true,
            xaxis: {
                autorange: true,
                rangeslider: {visible: false},
                showline: true,
                anchor: standaloneIndicators > 0 ? "y2" : "y1"
            },
            yaxis: {
                domain: standaloneIndicators > 0 ? [0.5, 1] : [0, 1],
                autorange: true,
                mirror: true,
                showline: true,
                gridcolor: "#ddd"
            },
            hovermode: "x unified",
            annotations: annotations,
        };

        let plotData = [candleStickData, buyData, sellData];
        const indicatorsHeight = 0.49/standaloneIndicators;
        let standaloneIndicatorIndex = 0;
        data.indicators.forEach((indicator) => {
            const axisNumber = standaloneIndicatorIndex+2;
            if (!indicator.overlay) {
                const heightStart = standaloneIndicatorIndex * indicatorsHeight;
                layout["yaxis"+axisNumber] = {
                    title: indicator.name,
                    domain: [heightStart, heightStart + indicatorsHeight],
                    autorange: true,
                    mirror: true,
                    showline: true,
                    linecolor: "black",
                    gridcolor: "#ddd"
                };
                standaloneIndicatorIndex++;
            }

            indicator.metrics.forEach(metric => {
                const data = {
                    title: indicator.name,
                    name: indicator.name + (metric.name && " - " + metric.name),
                    x: metric.time,
                    y: metric.value,
                    mode: metric.style,
                    line: {
                        color: metric.color,
                    },
                    xaxis: "x1",
                    yaxis: "y1",
                };
                if (!indicator.overlay) {
                    data.yaxis = "y"+axisNumber;
                }
                plotData.push(data);
            })
        });
        Plotly.newPlot("graph", plotData, layout);
    })
});
